/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert;

import java.io.PrintStream;
import java.io.PrintWriter;

/**
 * @author Ronald Blaschke
 */
public abstract class AbstractRuntimeException extends RuntimeException {
    private Throwable cause = this;

    public AbstractRuntimeException() {
    }

    public AbstractRuntimeException(final String message) {
        super(message);
    }

    public AbstractRuntimeException(final String message, final Throwable cause) {
        super(message);
        this.cause = cause;
    }

    public AbstractRuntimeException(final Throwable cause) {
        super(cause == null ? null : cause.toString());
        this.cause = cause;
    }

    public Throwable getCause() {
        return (cause == this ? null : cause);
    }

    public synchronized Throwable initCause(final Throwable cause) {
        if (this.cause != this)
            throw new IllegalStateException("Can't overwrite cause");
        if (cause == this)
            throw new IllegalArgumentException("Self-causation not permitted");
        this.cause = cause;
        return this;
    }

    public void printStackTrace() {
        synchronized (System.err) {
            printStackTrace(System.err);
        }
    }

    public void printStackTrace(final PrintStream s) {
        synchronized (s) {
            super.printStackTrace(s);

            final Throwable ourCause = getCause();
            if (ourCause != null) {
                s.println("Caused by: " + ourCause);
                ourCause.printStackTrace(s);
            }
        }
    }

    public void printStackTrace(final PrintWriter s) {
        synchronized (s) {
            super.printStackTrace(s);

            final Throwable ourCause = getCause();
            if (ourCause != null) {
                s.println("Caused by: " + ourCause);
                ourCause.printStackTrace(s);
            }
        }
    }
}
