/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert;

import org.rblasch.convert.graph.WeightedDirectedEdge;

public final class ConverterEdge implements WeightedDirectedEdge {
    private final MetaConverter converter;
    private final int weight;

    public ConverterEdge(final MetaConverter converter, final int weight) {
        this.converter = converter;
        this.weight = weight;
    }

    public MetaConverter getConverter() {
        return converter;
    }

    public int getWeight() {
        return weight;
    }

    public String toString() {
        return "-(" + weight + ")->";
    }


    // generated
    public boolean equals(final Object o) {
        if (this == o) return true;
        if (!(o instanceof ConverterEdge)) return false;

        final ConverterEdge converterEdge = (ConverterEdge) o;
        if (weight != converterEdge.weight) return false;
        if (!converter.equals(converterEdge.converter)) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = converter.hashCode();
        result = 29 * result + weight;
        return result;
    }
}
