/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert;

import org.rblasch.convert.converters.lang.CastConverter;
import org.rblasch.convert.converters.lang.IdentityConverter;
import org.rblasch.convert.converters.lang.ObjectToStringConverter;
import org.rblasch.convert.graph.Graph;
import org.rblasch.convert.graph.SparseWeightedDirectedGraph;
import org.rblasch.convert.graph.WeightedDirectedGraph;
import org.rblasch.convert.type.ClassType;
import org.rblasch.convert.type.ParameterizableType;
import org.rblasch.convert.type.Type;

import java.util.Arrays;
import java.util.Iterator;

/**
 * @author Ronald Blaschke
 * @deprecated no longer used
 */
public class JlGraphFactory {
    private static void addConnection(final Graph g, final MetaConverter mc) {
        g.addConnection(new TypeVertex(mc.getSourceType()),
                new ConverterEdge(mc, mc.getWeight()),
                new TypeVertex(mc.getDestinationType()));
    }

    public static WeightedDirectedGraph createGraphFor(final Type t) {
        final WeightedDirectedGraph g = new SparseWeightedDirectedGraph();

        // identity
        addConnection(g, new IdentityConverter(t));

        // class lookup
        if (t instanceof ClassType) {
            final ClassType ct = (ClassType) t;

            Class tmp = ct.getTheClass();
            while (tmp.getSuperclass() != null) {
                addConnection(g, new CastConverter(tmp, tmp.getSuperclass()));

                for (final Iterator i = Arrays.asList(tmp.getInterfaces()).iterator(); i.hasNext();) {
                    final Class iface = (Class) i.next();
                    addConnection(g, new CastConverter(tmp, iface));
                }

                tmp = tmp.getSuperclass();
            }
        } else if (t instanceof ParameterizableType) {
            //TODO: add parameter lookup
        }

        // "default" .toString() conversion
        addConnection(g, new ObjectToStringConverter());

        return g;
    }
}
