/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert.converters;

import org.rblasch.convert.ConverterFactory;
import org.rblasch.convert.ConverterLookup;
import org.rblasch.convert.MetaConverter;
import org.rblasch.convert.NoSuchConversionException;
import org.rblasch.convert.type.Type;

/**
 * @author Ronald Blaschke
 */
public abstract class AbstractConverterFactory implements ConverterFactory {
    ConverterLookup lookup;
    public void setLookup(final ConverterLookup lookup) {
        this.lookup = lookup;
    }

    protected MetaConverter lookup(final Type sourceType, final Type destinationType) {
        return lookup.lookup(sourceType, destinationType);
    }

    protected boolean can(final Type sourceType, final Type destinationType) {
        try {
            lookup.lookup(sourceType, destinationType);
            return true;
        } catch (final NoSuchConversionException ex) {
            return false;
        }
    }
}
