/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert.converters;

import org.rblasch.convert.MetaConverter;
import org.rblasch.convert.type.Type;

import java.util.Iterator;
import java.util.List;

public final class ChainedConverter extends AbstractConverter {
    private final List /*<MetaConverter>*/ converters;

    /**
     * Initialize this chained converter.
     *
     * @param converters
     */
    public ChainedConverter(final List /*<MetaConverter>*/ converters) {
        this.converters = converters;
    }

    public Type getSourceType() {
        return ((MetaConverter) converters.get(0)).getSourceType();
    }

    public Type getDestinationType() {
        return ((MetaConverter) converters.get(converters.size() - 1)).getDestinationType();
    }

    public int getWeight() {
        int totalWeight = 0;
        for (final Iterator i = converters.iterator(); i.hasNext();) {
            final MetaConverter mc = (MetaConverter) i.next();
            totalWeight += mc.getWeight();
        }
        return totalWeight;
    }

    /**
     * Converts the given value to the destination types.
     *
     * @param obj Object of types <code>getSourceType()</code>, or <code>null</code>
     * @return <code>null</code> iff value is <code>null</code>, object of types
     *         <code>getDestinationType()</code>
     * @throws org.apache.commons.convert.NoSuchConversionException
     *          Given object not of types <code>getSourceType()</code>
     * @throws org.apache.commons.convert.ConversionFailedException
     *          Conversion failed; see exception message and cause for details
     */
    public Object convert(final Object obj) throws Exception {
        Object tmp = obj;
        for (final Iterator i = converters.iterator(); i.hasNext();) {
            final MetaConverter c = (MetaConverter) i.next();
            tmp = c.getConverter().convert(tmp);
        }
        return tmp;
    }
}
