/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert.converters;

import org.rblasch.convert.Converter;
import org.rblasch.convert.MetaConverter;
import org.rblasch.convert.type.Type;
import org.rblasch.convert.type.Types;

/**
 * @author Ronald Blaschke
 */
public class ClassMetaConverter implements MetaConverter {
    private final Class sourceClass, destinationClass;
    private final Converter converter;
    private final int weight;

    public ClassMetaConverter(final Class sourceClass, final Class destinationClass, final Converter converter, final int weight) {
        this.sourceClass = sourceClass;
        this.destinationClass = destinationClass;
        this.converter = converter;
        this.weight = weight;
    }

    public Type getSourceType() {
        return Types.findTypeByClass(sourceClass);
    }

    public Type getDestinationType() {
        return Types.findTypeByClass(destinationClass);
    }

    public int getWeight() {
        return weight;
    }

    public Converter getConverter() {
        return converter;
    }
}
