/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert.converters;

import org.rblasch.convert.MetaConverter;
import org.rblasch.convert.type.Type;
import org.rblasch.convert.type.Types;

import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * @author Ronald Blaschke
 */
public class ObjectToMetaConvertersConverter extends AbstractConverter {
    private final int weight;

    public ObjectToMetaConvertersConverter(final int weight) {
        this.weight = weight;
    }

    public Type getSourceType() {
        return Types.findTypeByClass(Object.class);
    }

    public Type getDestinationType() {
        return Types.findPTypeByClass(Set.class, MetaConverter.class);
    }

    public int getWeight() {
        return 100;
    }

    public Object convert(final Object obj) throws Exception {
        final Set/*<MetaConverter>*/ converters = new HashSet();

        for (final Iterator i = Arrays.asList(obj.getClass().getMethods()).iterator(); i.hasNext();) {
            final Method m = (Method) i.next();
            if (m.getName().startsWith("convert")
                    && m.getParameterTypes().length == 1
                    && !m.getReturnType().equals(Void.TYPE)) {
                converters.add(new AbstractClassConverter() {
                    public Class getSourceClass() {
                        return m.getParameterTypes()[0];
                    }

                    public Class getDestinationClass() {
                        return m.getReturnType();
                    }

                    public int getWeight() {
                        return weight;
                    }

                    public Object convert(final Object arg) throws Exception {
                        return m.invoke(obj, new Object[]{arg});
                    }
                });
            }
        }

        return converters;
    }
}
