/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert.converters;

import org.rblasch.convert.type.Type;
import org.rblasch.convert.type.Types;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

/**
 * @author Ronald Blaschke
 */
public class StringToStringListConverter extends AbstractConverter {
    public Type getSourceType() {
        return Types.findTypeByClass(String.class);
    }

    public Type getDestinationType() {
        return Types.findPTypeByClass(List.class, String.class);
    }

    public int getWeight() {
        return 100;
    }

    public Object convert(final Object obj) throws Exception {
        final String s = (String) obj;

        final List/*<String>*/ l = new ArrayList();
        for (final StringTokenizer tokenizer = new StringTokenizer(s, ","); tokenizer.hasMoreTokens();) {
            l.add(tokenizer.nextToken());
        }
        
        return l;
    }
}
