/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert.converters.collections;

import org.rblasch.convert.ConversionFailedException;
import org.rblasch.convert.MetaConverter;
import org.rblasch.convert.assertion.Assertion;
import org.rblasch.convert.converters.AbstractConverter;
import org.rblasch.convert.type.ClassType;
import org.rblasch.convert.type.PrimitiveType;
import org.rblasch.convert.type.Type;
import org.rblasch.convert.type.Types;

import java.lang.reflect.Array;

/**
 * @author Ronald Blaschke
 */
public class ArrayConverter extends AbstractConverter {
    private final Type sourceCType, destinationCType;
    private final MetaConverter cConverter;

    public ArrayConverter(final Type sourceCType, final Type destinationCType, final MetaConverter cConverter) {
        this.sourceCType = sourceCType;
        this.destinationCType = destinationCType;
        this.cConverter = cConverter;
    }

    public Type getSourceType() {
        return Types.findPTypeByType(Types.findTypeByName("array"), sourceCType);
    }

    public Type getDestinationType() {
        return Types.findPTypeByType(Types.findTypeByName("array"), destinationCType);
    }

    public int getWeight() {
        return 100;
    }

    public Object convert(final Object obj) throws Exception {
        Assertion.ensure(sourceCType instanceof ClassType
        || sourceCType instanceof PrimitiveType, "source component must be class or primitive");
        Assertion.ensure(destinationCType instanceof ClassType
        || destinationCType instanceof PrimitiveType, "destination component must be class or primitive");

        final Object in = obj;
        final Class outComponentClass;
        if (destinationCType instanceof ClassType) {
            outComponentClass = ((ClassType) destinationCType).getTheClass();
        } else if (destinationCType instanceof PrimitiveType) {
            outComponentClass = ((PrimitiveType) destinationCType).getTheClass();
        } else {
            throw new ConversionFailedException("Illegal source component type " + sourceCType);
        }

        final Object out = Array.newInstance(outComponentClass, Array.getLength(in));
        for (int i = 0; i < Array.getLength(in); ++i) {
            Array.set(out, i, cConverter.getConverter().convert(Array.get(in, i)));
        }

        return out;
    }
}
