/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert.converters.collections;

import org.rblasch.convert.MetaConverter;
import org.rblasch.convert.converters.AbstractConverter;
import org.rblasch.convert.type.Type;
import org.rblasch.convert.type.Types;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Ronald Blaschke
 */
public class ArrayToArrayListConverter extends AbstractConverter {
    private final Type sourceCType, destinationCType;
    private final MetaConverter cConverter;

    public ArrayToArrayListConverter(final Type sourceCType, final Type destinationCType, final MetaConverter cConverter) {
        this.sourceCType = sourceCType;
        this.destinationCType = destinationCType;
        this.cConverter = cConverter;
    }

    public Type getSourceType() {
        return Types.findPTypeByType(Types.primitiveArrayType(), sourceCType);
    }

    public Type getDestinationType() {
        return Types.findPTypeByType(Types.findTypeByClass(ArrayList.class), destinationCType);
    }

    public int getWeight() {
        return 100;
    }

    public Object convert(final Object obj) throws Exception {
        final Object[] array = (Object[]) obj;
        final List l = new ArrayList(array.length);
        for (int i = 0; i < array.length; ++i) {
            l.add(cConverter.getConverter().convert(array[i]));
        }
        return l;
    }
}
