/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert.converters.collections;

import org.rblasch.convert.converters.AbstractConverterFactory;
import org.rblasch.convert.type.ParameterizableType;
import org.rblasch.convert.type.Type;
import org.rblasch.convert.type.Types;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * @author Ronald Blaschke
 */
public class ArrayToListConverterFactory extends AbstractConverterFactory {
    public Set/*<MetaConverter>*/ create(final Type sourceType, final Type destinationType) {
        final Set/*<MetaConverter>*/ converters = new HashSet();

        if (sourceType instanceof ParameterizableType
                && destinationType instanceof ParameterizableType) {
            final ParameterizableType st = (ParameterizableType) sourceType;
            final ParameterizableType dt = (ParameterizableType) destinationType;

            if (st.getType().equals(Types.primitiveArrayType())
                    && dt.getType().equals(Types.findTypeByClass(List.class))) {
                final Type sct = st.getParameter(0);
                final Type dct = dt.getParameter(0);

                if (can(sct, dct)) {
                    final ArrayToArrayListConverter c = new ArrayToArrayListConverter(sct, dct, lookup(sct, dct));
                    converters.add(c);
                }
            }
        }

        return converters;
    }
}
