/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert.converters.primitives;

import org.rblasch.convert.ConversionFailedException;
import org.rblasch.convert.converters.AbstractClassConverter;

/**
 * @author Ronald Blaschke
 */
public class IntegerToByteConverter extends AbstractClassConverter {
    public Class getSourceClass() {
        return Integer.class;
    }

    public Class getDestinationClass() {
        return Byte.class;
    }

    public int getWeight() {
        return 100;
    }

    public Object convert(final Object value) throws ConversionFailedException {
        final Integer in = (Integer) value;

        if (in.intValue() < Byte.MIN_VALUE || in.intValue() > Byte.MAX_VALUE) {
            throw new ConversionFailedException("Input is out of range ["
                    + Byte.MIN_VALUE + ";" + Byte.MAX_VALUE + "]: " + in);
        }

        return new Byte(in.byteValue());
    }
}
