/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert.converters.primitives;

import org.rblasch.convert.ConversionFailedException;
import org.rblasch.convert.converters.AbstractClassConverter;

/**
 * @author Ronald Blaschke
 */
public class ShortToByteConverter extends AbstractClassConverter {
    public Class getSourceClass() {
        return Short.class;
    }

    public Class getDestinationClass() {
        return Byte.class;
    }

    public int getWeight() {
        return 100;
    }

    public Object convert(final Object value) throws ConversionFailedException {
        final Short s = (Short) value;

        if (s.shortValue() < Byte.MIN_VALUE || s.shortValue() > Byte.MAX_VALUE) {
            throw new ConversionFailedException("Input is out of range ["
                    + Byte.MIN_VALUE + ";" + Byte.MAX_VALUE + "]: " + s);
        }

        return new Byte(s.byteValue());
    }
}
