/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert.converters.primitives;

import org.rblasch.convert.ConversionFailedException;
import org.rblasch.convert.converters.AbstractClassConverter;

/**
 * @author Ronald Blaschke
 */
public class StringToBooleanConverter extends AbstractClassConverter {
    public Class getSourceClass() {
        return String.class;
    }

    public Class getDestinationClass() {
        return Boolean.class;
    }

    public int getWeight() {
        return 100;
    }

    private static boolean isTrue(final String s) {
        return "true".equalsIgnoreCase(s)
                || "1".equalsIgnoreCase(s);
    }

    private static boolean isFalse(final String s) {
        return "false".equalsIgnoreCase(s)
                || "0".equalsIgnoreCase(s);
    }

    public Object convert(final Object obj) throws Exception {
        final String s = ((String) obj).trim();
        if (isTrue(s)) {
            return Boolean.TRUE;
        } else if (isFalse(s)) {
            return Boolean.FALSE;
        } else {
            throw new ConversionFailedException("Can't convert '" + obj + "' to boolean");
        }
    }
}
