/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert.graph;

public final class Connection {
    private final Vertex start, end;
    private final Edge edge;

    public Connection(final Vertex start, final Edge edge, final Vertex end) {
        this.start = start;
        this.edge = edge;
        this.end = end;
    }

    public Vertex getStart() {
        return start;
    }

    public Edge getEdge() {
        return edge;
    }

    public Vertex getEnd() {
        return end;
    }

    public String toString() {
        return start + " " + edge + " " + end;
    }

    // generated
    public boolean equals(final Object o) {
        if (this == o) return true;
        if (!(o instanceof Connection)) return false;

        final Connection connection = (Connection) o;

        if (!edge.equals(connection.edge)) return false;
        if (!end.equals(connection.end)) return false;
        if (!start.equals(connection.start)) return false;

        return true;
    }

    public int hashCode() {
        int result;
        result = start.hashCode();
        result = 29 * result + end.hashCode();
        result = 29 * result + edge.hashCode();
        return result;
    }
}
