/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert.graph;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class ListPath implements Path {
    private final List /* of Vertex, Edge */ path;

    public ListPath(final List /* of Vertex, Edge */ path) {
        this.path = path;
    }

    public Vertex getStart() {
        return (Vertex) path.get(0);
    }

    public Vertex getEnd() {
        return (Vertex) path.get(path.size() - 1);
    }

    public List /* of Vertex */ getVertices() {
        final List /* of Vertex */ vertices = new LinkedList();
        for (final Iterator i = path.iterator(); i.hasNext();) {
            final Object n = i.next();
            if (n instanceof Vertex) {
                vertices.add(n);
            }
        }
        return vertices;
    }

    public List /* of Edge */ getEdges() {
        final List /* of Edge */ edges = new LinkedList();
        for (final Iterator i = path.iterator(); i.hasNext();) {
            final Object n = i.next();
            if (n instanceof Edge) {
                edges.add(n);
            }
        }
        return edges;
    }

    public Iterator /* of Vertex, Edge */ iterator() {
        return path.iterator();
    }
}
