package org.apache.commons.convert;

import org.apache.commons.convert.converters.IdentityConverter;
import org.apache.commons.convert.converters.ObjectToStringConverter;

import java.util.Iterator;

public class InheritanceLookup implements ConverterLookup {
    private final ConverterRegistry registry;

    public InheritanceLookup(final ConverterRegistry registry) {
        this.registry = registry;
    }

    public Converter lookup(final Class sourceType, final Class destinationType) throws NoSuchConversionException {
        // identity
        if (destinationType.isAssignableFrom(sourceType)) {
            return new IdentityConverter(destinationType);
        }

        // lookup
        for (final Iterator i = registry.iterator(); i.hasNext();) {
            final Converter c = (Converter) i.next();
            if (sourceType.isAssignableFrom(c.getSourceClass()) && c.getDestinationClass().isAssignableFrom(destinationType)) {
                return c;
            }
        }

        // "default" .toString() conversion
        if (destinationType.equals(String.class)) {
            return new ObjectToStringConverter();
        }
        throw new NoSuchConversionException("No conversion from " + sourceType + " to " + destinationType);
    }
}
