package org.apache.commons.convert.converters;

import org.apache.commons.convert.ConversionFailedException;
import org.apache.commons.convert.NoSuchConversionException;

public final class IdentityConverter extends AbstractConverter {
    private final Class type;

    /**
     * Initialize identity converter with the given source and destination types, which must
     * be equal.
     *
     * @param type
     */
    public IdentityConverter(final Class type) {
        this.type = type;
    }

    /**
     * Source types for conversion.
     * <p/>
     * <p>The input value for <code>convert</code> must be of this types.</p>
     *
     * @return Source types for conversion, not <code>null</code>
     */
    public Class getSourceClass() {
        return type;
    }

    /**
     * Destination types for conversion.
     * <p/>
     * <p>The return value of <code>convert</code> will be of this types.</p>
     *
     * @return Destination types for conversion, not <code>null</code>
     */
    public Class getDestinationClass() {
        return type;
    }

    public Object convert(final Object value) throws NoSuchConversionException, ConversionFailedException {
        return value;
    }
}
