package org.apache.commons.convert.extended;

import org.apache.commons.convert.converters.ObjectToStringConverter;
import org.apache.commons.convert.extended.converters.ExtendedCastConverter;
import org.apache.commons.convert.extended.types.ArrayType;

import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public final class ClassExtendedConverterRegistry extends AbstractExtendedConverterRegistry {
    private final List /* of ExtendedConverter */ converters;

    public ClassExtendedConverterRegistry(final Class clazz) {
        converters = createConverters(clazz);
    }

    public Iterator /* of ExtendedConverter */ iterator() {
        return converters.iterator();
    }

    public String toString() {
        return converters.toString();
    }

    private List /* of ExtendedConverter */ createConverters(final Class clazz) {
        final List /* of ExtendedConverter */ converters = new LinkedList();
        converters.add(new ConverterAdapter(new ObjectToStringConverter()));

        {
            final List /* of Class */ resolvedClasses = new LinkedList();
            final List /* of Class */ pendingClasses = new LinkedList();
            pendingClasses.add(clazz);

            while (!pendingClasses.isEmpty()) {
                final Class c = (Class) pendingClasses.remove(0);

                if (!resolvedClasses.contains(c)) {
                    resolvedClasses.add(c);
                    final Class superclass = c.getSuperclass();
                    if (superclass != null) {
                        pendingClasses.add(superclass);
                        converters.add(new ExtendedCastConverter(Types.toType(c), Types.toType(superclass)));
                    }
                    for (final Iterator i = Arrays.asList(c.getInterfaces()).iterator(); i.hasNext();) {
                        final Class iface = (Class) i.next();
                        pendingClasses.add(iface);
                        converters.add(new ExtendedCastConverter(Types.toType(c), Types.toType(iface)));
                    }
                }
            }
        }

        if (clazz.isArray()) {
            final List /* of Class */ resolvedClasses = new LinkedList();
            final List /* of Class */ pendingClasses = new LinkedList();
            pendingClasses.add(clazz.getComponentType());

            while (!pendingClasses.isEmpty()) {
                final Class c = (Class) pendingClasses.remove(0);

                if (!resolvedClasses.contains(c)) {
                    resolvedClasses.add(c);
                    final Class superclass = c.getSuperclass();
                    if (superclass != null) {
                        pendingClasses.add(superclass);
                        converters.add(new ExtendedCastConverter(Types.toType(ArrayType.class, c), Types.toType(ArrayType.class, superclass)));
                    }
                    for (final Iterator i = Arrays.asList(c.getInterfaces()).iterator(); i.hasNext();) {
                        final Class iface = (Class) i.next();
                        pendingClasses.add(iface);
                        converters.add(new ExtendedCastConverter(Types.toType(ArrayType.class, c), Types.toType(ArrayType.class, iface)));
                    }
                }
            }
        }

        return converters;
    }
}
