package org.apache.commons.convert.extended;

import org.apache.commons.convert.ConversionFailedException;
import org.apache.commons.convert.Converter;
import org.apache.commons.convert.NoSuchConversionException;

public final class ConverterAdapter implements ExtendedConverter {
    private Converter converter;

    public ConverterAdapter(final Converter converter) {
        this.converter = converter;
    }

    public Type getSourceType() {
        return Types.toType(converter.getSourceClass());
    }

    public Type getDestinationType() {
        return Types.toType(converter.getDestinationClass());
    }

    public ConversionContext convert(final ConversionContext context) throws NoSuchConversionException, ConversionFailedException {
        return context.update(getDestinationType(), converter.convert(context.getValue()));
    }

    // generated
    public boolean equals(final Object o) {
        if (this == o) return true;
        if (!(o instanceof ConverterAdapter)) return false;

        final ConverterAdapter converterAdapter = (ConverterAdapter) o;

        if (!converter.equals(converterAdapter.converter)) return false;

        return true;
    }

    public int hashCode() {
        return converter.hashCode();
    }
}
