/*
 * $Header$
 * $Revision$
 * $Date$
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999-2004 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowledgement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgement may appear in the software itself,
 *    if and wherever such third-party acknowledgements normally appear.
 *
 * 4. The names "The Jakarta Project", "Commons", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.commons.convert.extended;

import org.apache.commons.convert.ConversionFailedException;
import org.apache.commons.convert.NoSuchConversionException;
import org.apache.commons.convert.converters.*;

public final class ExtendedStandardConverter {
    private static final ExtendedStandardConverter INSTANCE = createDefaultStandardConverter();

    public static ExtendedStandardConverter getInstance() {
        return INSTANCE;
    }

    private static ExtendedConverterRegistry createDefaultRegistry() {
        final MutableExtendedConverterRegistry registry = new MutableExtendedConverterRegistry();
        registry.add(new IntegerToByteConverter());
        registry.add(new StringToIntegerConverter());

        // just for testing, should not be standard (I guess...)
        registry.add(new ByteToBooleanConverter());

        registry.add(new IntArrayToIntegerArrayConverter());
        registry.add(new ArrayToArrayListConverter());
        return registry;
    }

    private static ExtendedStandardConverter createDefaultStandardConverter() {
        return new ExtendedStandardConverter(createDefaultRegistry());
    }

    private final MutableExtendedConverterRegistry registry = new MutableExtendedConverterRegistry();

    public ExtendedStandardConverter() {
        registry.addAll(createDefaultRegistry());
    }

    public ExtendedStandardConverter(final ExtendedConverterRegistry registry) {
        this.registry.addAll(registry);
    }

    public void addConverter(final ExtendedConverter converter) {
        registry.add(converter);
    }

    public void removeConverter(final ExtendedConverter converter) {
        registry.remove(converter);
    }

    public Object convert(final Object value, final Class destinationType) throws NoSuchConversionException, ConversionFailedException {
        return convert(value, Types.toType(destinationType));
    }

    public Object convert(final Object value, final Type destinationType) throws NoSuchConversionException, ConversionFailedException {
        if (value == null) {
            return null;
        }
        return new ShortestPathLookup(registry).lookup(Types.typeOf(value), destinationType).convert(new ImmutableConversionContext(Types.typeOf(value), value)).getValue();
    }
}
