package org.apache.commons.convert.extended;

import org.apache.commons.convert.NoSuchConversionException;
import org.apache.commons.convert.extended.converters.ChainedExtendedConverter;
import org.rblasch.convert.dijkstra.ShortestPathEngine;
import org.rblasch.convert.graph.Graph;
import org.rblasch.convert.graph.Path;
import org.rblasch.convert.graph.SparseWeightedDirectedGraph;
import org.rblasch.convert.graph.WeightedDirectedGraph;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class ShortestPathLookup implements ExtendedConverterLookup {
    private final WeightedDirectedGraph graph;

    public ShortestPathLookup(final ExtendedConverterRegistry registry) {
        graph = new SparseWeightedDirectedGraph();
        for (final Iterator i = registry.iterator(); i.hasNext();) {
            final ExtendedConverter converter = (ExtendedConverter) i.next();
            graph.addConnection(new TypeVertex(converter.getSourceType()), new ConverterEdge(converter, 1), new TypeVertex(converter.getDestinationType()));

            addTypeConverters(graph, converter.getSourceType());
            addTypeConverters(graph, converter.getDestinationType());
        }
    }

    private void addTypeConverters(final Graph graph, final Type type) {
        for (final Iterator i = type.getConverters().iterator(); i.hasNext();) {
            final ExtendedConverter c = (ExtendedConverter) i.next();
            graph.addConnection(new TypeVertex(c.getSourceType()), new ConverterEdge(c, 1), new TypeVertex(c.getDestinationType()));
        }
    }

    private List /* of ExtendedConverter */ getConvertersForPath(final Path path) {
        final List /* of ExtendedConverter */ converters = new LinkedList();

        for (final Iterator i = path.getEdges().iterator(); i.hasNext();) {
            final ExtendedConverter c = ((ConverterEdge) i.next()).getConverter();
            converters.add(c);
        }

        return converters;
    }

    private Path getShortestPath(final Type sourceType, final Type destinationType) throws NoSuchConversionException {
        final ShortestPathEngine spEngine = new ShortestPathEngine(graph);
        final Path shortestPath = spEngine.getShortestPath(new TypeVertex(sourceType), new TypeVertex(destinationType));
        if (shortestPath == null) {
            throw new NoSuchConversionException("No conversion from " + sourceType + " to " + destinationType + " in " + graph);
        }
        return shortestPath;
    }

    public ExtendedConverter lookup(final Type sourceType, final Type destinationType) throws NoSuchConversionException {
        addTypeConverters(graph, sourceType);
        addTypeConverters(graph, destinationType);
        return new ChainedExtendedConverter(getConvertersForPath(getShortestPath(sourceType, destinationType)));
    }
}
