package org.apache.commons.convert.extended;

import org.apache.commons.convert.extended.ClassExtendedConverterRegistry;
import org.apache.commons.convert.extended.ExtendedConverterRegistry;
import org.apache.commons.convert.extended.Type;
import org.apache.commons.convert.extended.types.ArrayType;
import org.apache.commons.convert.extended.types.ClassType;

import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public final class Types {
    private Types() {
    }

    public static Type typeOf(final Object obj) {
        return toType(obj == null ? null : obj.getClass());
    }

    public static Type toType(final Class clazz) {
        return toType(clazz, new Class[]{});
    }

    public static Type toType(final Class clazz, final Class param1) {
        return toType(clazz, new Class[]{param1});
    }

    public static Type toType(final Class clazz, final Class[] parameters) {
        final String name;
        final List /* of Type */ typeParameters = new LinkedList();
        if (clazz.isArray()) {
            //name = (clazz.getComponentType().isPrimitive() ? clazz.getComponentType().getName() : Object.class.getName()) + "[]";
            name = ArrayType.class.getName();
            typeParameters.add(toType(clazz.getComponentType()));
        } else {
            name = clazz.getName();
            for (int i = 0; i < parameters.length; ++i) {
                typeParameters.add(toType(parameters[i]));
            }
        }

        return new ClassType(name, typeParameters, clazz);
    }

    private static ExtendedConverterRegistry converterRegistryOf(final Class clazz) {
        return new ClassExtendedConverterRegistry(clazz);
    }

    private static List /* of Type */ toAllTypes(final Class clazz) {
        final List /* of Type */ types = new LinkedList();

        final List /* of Class */ resolvedClasses = new LinkedList();
        final List /* of Class */ pendingClasses = new LinkedList();
        pendingClasses.add(clazz);

        while (!pendingClasses.isEmpty()) {
            final Class c = (Class) pendingClasses.remove(0);
            if (!resolvedClasses.contains(c)) {
                resolvedClasses.add(c);
                if (clazz.getSuperclass() != null) {
                    pendingClasses.add(clazz.getSuperclass());
                }
                pendingClasses.addAll(Arrays.asList(clazz.getInterfaces()));
            }
        }

        for (final Iterator i = resolvedClasses.iterator(); i.hasNext();) {
            types.add(toType((Class) i.next()));
        }

        return types;
    }
}
