package org.apache.commons.convert.extended.converters;

import org.apache.commons.convert.ConversionFailedException;
import org.apache.commons.convert.NoSuchConversionException;
import org.apache.commons.convert.extended.ConversionContext;
import org.apache.commons.convert.extended.ExtendedConverter;

public abstract class AbstractExtendedConverter implements ExtendedConverter {
    protected abstract ConversionContext convertInternal(final ConversionContext context) throws NoSuchConversionException, ConversionFailedException;

    public final ConversionContext convert(final ConversionContext context) throws NoSuchConversionException, ConversionFailedException {
        if (context.getValue() == null) {
            return context;
        }

        if (!getSourceType().equals(context.getType())) {
            throw new NoSuchConversionException("Can't convert context of types " + context.getType()
                    + "; only " + getSourceType() + " is supported");
        }

        final ConversionContext result = convertInternal(context);

        if (result.getValue() == null) {
            throw new ConversionFailedException("ExtendedConverter implementation returned null");
        }

        if (!getDestinationType().equals(result.getType())) {
            throw new ConversionFailedException("Conversion returned illegal types "
                    + result.getType() + ", but expected " + getDestinationType());
        }

        return result;
    }

    public final String toString() {
        return getSourceType() + " -> " + getDestinationType();
    }
}
