package org.apache.commons.convert.extended.converters;

import org.apache.commons.convert.ConversionFailedException;
import org.apache.commons.convert.NoSuchConversionException;
import org.apache.commons.convert.extended.ConversionContext;
import org.apache.commons.convert.extended.ExtendedConverter;
import org.apache.commons.convert.extended.Type;

import java.util.Iterator;
import java.util.List;

public final class ChainedExtendedConverter implements ExtendedConverter {
    private final List /* of ExtendedConverter */ converters;

    /**
     * Initialize this chained converter.
     *
     * @param converters
     */
    public ChainedExtendedConverter(final List /* of ExtendedConverter */ converters) {
        this.converters = converters;
    }

    /**
     * Source types for conversion.
     * <p/>
     * <p>The input value for <code>convert</code> must be of this types.</p>
     *
     * @return Source types for conversion, not <code>null</code>
     */
    public Type getSourceType() {
        return ((ExtendedConverter) converters.get(0)).getSourceType();
    }

    /**
     * Destination types for conversion.
     * <p/>
     * <p>The return value of <code>convert</code> will be of this types.</p>
     *
     * @return Destination types for conversion, not <code>null</code>
     */
    public Type getDestinationType() {
        return ((ExtendedConverter) converters.get(converters.size() - 1)).getDestinationType();
    }

    /**
     * Converts the given value to the destination types.
     *
     * @param context Object of types <code>getSourceType()</code>, or <code>null</code>
     * @return <code>null</code> iff value is <code>null</code>, object of types
     *         <code>getDestinationType()</code>
     * @throws org.apache.commons.convert.NoSuchConversionException
     *          Given object not of types <code>getSourceType()</code>
     * @throws org.apache.commons.convert.ConversionFailedException
     *          Conversion failed; see exception message and cause for details
     */
    public ConversionContext convert(final ConversionContext context) throws NoSuchConversionException, ConversionFailedException {
        ConversionContext tmp = context;
        for (final Iterator i = converters.iterator(); i.hasNext();) {
            final ExtendedConverter c = (ExtendedConverter) i.next();
            tmp = c.convert(tmp);
        }
        return tmp;
    }
}
