package org.apache.commons.convert.extended.converters;

import org.apache.commons.convert.ConversionFailedException;
import org.apache.commons.convert.NoSuchConversionException;
import org.apache.commons.convert.extended.ConversionContext;
import org.apache.commons.convert.extended.Type;

public final class ExtendedIdentityConverter extends AbstractExtendedConverter {
    private final Type type;

    /**
     * Initialize identity converter with the given source and destination types, which must
     * be equal.
     *
     * @param type
     */
    public ExtendedIdentityConverter(final Type type) {
        this.type = type;
    }

    /**
     * Source types for conversion.
     * <p/>
     * <p>The input value for <code>convert</code> must be of this types.</p>
     *
     * @return Source types for conversion, not <code>null</code>
     */
    public Type getSourceType() {
        return type;
    }

    /**
     * Destination types for conversion.
     * <p/>
     * <p>The return value of <code>convert</code> will be of this types.</p>
     *
     * @return Destination types for conversion, not <code>null</code>
     */
    public Type getDestinationType() {
        return type;
    }

    protected ConversionContext convertInternal(final ConversionContext context) throws NoSuchConversionException, ConversionFailedException {
        return context.update(getDestinationType(), context.getValue());
    }
}
