package org.apache.commons.convert;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.util.LinkedList;
import java.util.List;

public final class StandardConverterTest extends TestCase {
    public static void main(final String[] argv) {
        junit.textui.TestRunner.run(suite());
    }

    public static Test suite() {
        final TestSuite suite = new TestSuite();
        suite.addTestSuite(StandardConverterTest.class);
        return suite;
    }

    public StandardConverterTest(final String name) {
        super(name);
    }

    /**
     * Tests for trivial conversions, ie conversions to the same types.
     *
     * @throws Exception
     */
    public void testConvertTrivial() throws Exception {
        assertEquals(null,
                StandardConverter.getInstance().convert(null, String.class));

        assertEquals("foo",
                StandardConverter.getInstance().convert("foo", String.class));

        assertEquals(new Integer(42),
                StandardConverter.getInstance().convert(new Integer(42), Integer.class));
    }

    /**
     * Simple conversions, implemented directly.
     *
     * @throws Exception
     */
    public void testConvertSimple() throws Exception {
        assertEquals(new Byte((byte) 42),
                StandardConverter.getInstance().convert(new Integer(42), Byte.class));

        assertEquals(new Integer(42),
                StandardConverter.getInstance().convert("42", Integer.class));
    }

    public void testConvertInterface() throws Exception {
        final LinkedList l = new LinkedList();
        l.add(new Integer(42));
        assertEquals(l, StandardConverter.getInstance().convert(l, List.class));
    }

    public void testIntArray() throws Exception {
        final int[] array = new int[2];
        array[0] = 42;
        array[1] = 43;

        final Integer[] array2 = (Integer[]) StandardConverter.getInstance().convert(array, Integer[].class);
        assertEquals(array[0], array2[0].intValue());
        assertEquals(array[1], array2[1].intValue());
    }

    /**
     * No special Short -> String converter; rely on <code>.toString()</code>.
     *
     * @throws Exception
     */
    public void testDefaultToString() throws Exception {
        assertEquals("42", StandardConverter.getInstance().convert(new Short((short)42), String.class));
    }

    public void testNull() throws Exception {
        assertEquals("null", StandardConverter.getInstance().convert(null, Integer.class,  String.class));
    }

    /**
     * Chain up multiple converters.
     *
     * @throws Exception
     */
    public void testConvertTransitive() throws Exception {
        // no direct conversion from String to Byte, but possible via String -> Integer -> Byte conversion
        try {
            StandardConverter.getInstance().convert("42", Byte.class);
            fail("Should throw NoSuchConversionException");
        } catch (final NoSuchConversionException ex) {
            assertTrue("Expected", true);
        }
    }
}
