package org.apache.commons.convert.extended;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.apache.commons.convert.extended.Types;

import java.util.LinkedList;
import java.util.List;

public final class ExtendedStandardConverterTest extends TestCase {
    public static void main(final String[] argv) {
        junit.textui.TestRunner.run(suite());
    }

    public static Test suite() {
        final TestSuite suite = new TestSuite();
        suite.addTestSuite(ExtendedStandardConverterTest.class);
        return suite;
    }

    public ExtendedStandardConverterTest(final String name) {
        super(name);
    }

    /**
     * Tests for trivial conversions, ie conversions to the same types.
     *
     * @throws Exception
     */
    public void testConvertTrivial() throws Exception {
        assertEquals(null,
                ExtendedStandardConverter.getInstance().convert(null, String.class));

        assertEquals("foo",
                ExtendedStandardConverter.getInstance().convert("foo", String.class));

        assertEquals(new Integer(42),
                ExtendedStandardConverter.getInstance().convert(new Integer(42), Integer.class));
    }

    /**
     * Simple conversions, implemented directly.
     *
     * @throws Exception
     */
    public void testConvertSimple() throws Exception {
        assertEquals(new Byte((byte) 42),
                ExtendedStandardConverter.getInstance().convert(new Integer(42), Byte.class));

        assertEquals(new Integer(42),
                ExtendedStandardConverter.getInstance().convert("42", Integer.class));
    }

    public void testConvertInterface() throws Exception {
        final LinkedList l = new LinkedList();
        l.add(new Integer(42));
        assertEquals(l, ExtendedStandardConverter.getInstance().convert(l, Types.toType(List.class)));
    }

    /**
     * Chain up multiple converters.
     *
     * @throws Exception
     */
    public void testConvertTransitive() throws Exception {
        // no direct conversion from String to Byte; takes String -> Integer -> Byte conversion
        assertEquals(new Byte((byte) 42),
                ExtendedStandardConverter.getInstance().convert("42", Byte.class));

        // just for testing, should not be standard (I guess...): String -> Integer -> Byte -> Boolean
        assertEquals(Boolean.TRUE,
                ExtendedStandardConverter.getInstance().convert("42", Boolean.class));

        assertEquals(Boolean.FALSE,
                ExtendedStandardConverter.getInstance().convert("0", Boolean.class));
    }

    public void testIntArray() throws Exception {
        final int[] array = new int[2];
        array[0] = 42;
        array[1] = 43;

        final Integer[] array2 = (Integer[]) ExtendedStandardConverter.getInstance().convert(array, Integer[].class);
        assertEquals(array[0], array2[0].intValue());
        assertEquals(array[1], array2[1].intValue());
    }

    public void testArrayList() throws Exception {
        final int[] array = new int[2];
        array[0] = 42;
        array[1] = 43;

        final List list = (List) ExtendedStandardConverter.getInstance().convert(array, List.class);
        assertEquals(array[0], ((Integer) list.get(0)).intValue());
        assertEquals(array[1], ((Integer) list.get(1)).intValue());
    }

    public void testParamType() throws Exception {
        try {
            final int[] array = new int[2];
            array[0] = 42;
            array[1] = 43;

            final List list = (List) ExtendedStandardConverter.getInstance().convert(array, Types.toType(List.class, Integer.class));
            assertEquals(array[0], ((Integer) list.get(0)).intValue());
            assertEquals(array[1], ((Integer) list.get(1)).intValue());

            fail("Should not be working (yet)");
        } catch (final Throwable up) {
            ; // expected
        }
    }

    public void testParamType2() throws Exception {
        try {
            final int[] array = new int[2];
            array[0] = 42;
            array[1] = 43;

            final List list = (List) ExtendedStandardConverter.getInstance().convert(array, Types.toType(List.class, Byte.class));
            assertEquals(array[0], ((Byte) list.get(0)).byteValue());
            assertEquals(array[1], ((Byte) list.get(1)).byteValue());

            fail("Should not be working (yet)");
        } catch (final Throwable up) {
            ;  // expected
        }
    }
}
