/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert;

import junit.framework.TestCase;
import org.rblasch.convert.converters.StringToStringListConverter;
import org.rblasch.convert.converters.collections.ArrayConverterFactory;
import org.rblasch.convert.converters.lang.JlConverterFactory;
import org.rblasch.convert.converters.primitives.StringToIntegerConverter;
import org.rblasch.convert.type.Types;

import java.util.List;

/**
 * @author Ronald Blaschke
 */
public class ArrayTest extends TestCase {
    public void testPrimitive() throws Exception {
        final SptConverter ci = SptConverter.createEmpty();
        ci.addFactory(new JlConverterFactory());
        ci.addFactory(new ArrayConverterFactory());
        final byte[] bytes = new byte[] {1, 2, 3};

        final Object o = ci.convert(bytes, Types.findTypeByName("array<int>"));
        assertEquals(int[].class, o.getClass());
    }

    public void testStringToStringList2() {
        final SptConverter ci = SptConverter.createDefault();
        ci.addConverter(new StringToStringListConverter());
        ci.addConverter(new StringToIntegerConverter());

        ci.convert("10", Types.findTypeByClass(Integer.class));
        final List/*<Integer>*/ l = (List) ci.convert("1,42,3", Types.findPTypeByClass(List.class, Integer.class));
        assertEquals(3, l.size());
        assertEquals(new Integer(1), l.get(0));
        assertEquals(new Integer(42), l.get(1));
        assertEquals(new Integer(3), l.get(2));
        //System.out.println(ci.dumpGraph());
    }
}
