/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert;

import junit.framework.TestCase;
import org.rblasch.convert.converters.primitives.StringToIntegerConverter;
import org.rblasch.convert.type.Types;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Ronald Blaschke
 */
public class PConverterTest extends TestCase {
    public void testList() throws Exception {
        final SptConverter ci = SptConverter.createDefault();
        final List/*<Integer>*/ iList = new ArrayList();
        iList.add(new Integer(42));

        final List/*<String>*/ sList = new ArrayList();
        sList.add("42");

        final Object l = ci.convert(iList, Types.findPTypeByType(Types.findTypeByClass(List.class), Types.findTypeByClass(Integer.class)), Types.findPTypeByType(Types.findTypeByClass(List.class), Types.findTypeByClass(String.class)));
        assertEquals(sList, l);
    }

    public void testParameterized() throws Exception {
        final SptConverter ci = SptConverter.createDefault();

        final Integer[] array = new Integer[1];
        array[0] = new Integer(42);

        final Object l = ci.convert(array, Types.findPTypeByType(Types.findTypeByClass(List.class), Types.findTypeByClass(String.class)));
        assertTrue(l instanceof List);
        assertEquals("42", ((List) l).get(0));
    }

    public void testStringListToInteger() throws Exception {
        final SptConverter ci = SptConverter.createDefault();
        ci.addConverter(new StringToIntegerConverter());
        final List/*<String>*/ sList = new ArrayList();
        sList.add("42");

        final List/*<Integer>*/ iList = new ArrayList();
        iList.add(new Integer(42));

        final Object l = ci.convert(sList, Types.findPTypeByType(Types.findTypeByClass(List.class), Types.findTypeByClass(String.class)), Types.findPTypeByType(Types.findTypeByClass(List.class), Types.findTypeByClass(Integer.class)));
        assertEquals(iList, l);
    }
}
