/*
 * Copyright 2004 Ronald Blaschke.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.rblasch.convert.dijkstra;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.rblasch.convert.graph.Edge;
import org.rblasch.convert.graph.ListPath;
import org.rblasch.convert.graph.Path;
import org.rblasch.convert.graph.Vertex;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class ListPathTest extends TestCase {
    public static void main(final String[] argv) {
        junit.textui.TestRunner.run(suite());
    }

    public static Test suite() {
        final TestSuite suite = new TestSuite();
        suite.addTestSuite(ListPathTest.class);
        return suite;
    }

    public ListPathTest(final String name) {
        super(name);
    }

    public void testTrivialPath() throws Exception {
        final Vertex v = new TestVertex();
        final List pathList = new LinkedList();
        pathList.add(v);
        final Path path = new ListPath(pathList);

        assertEquals(v, path.getStart());
        assertEquals(v, path.getEnd());

        assertTrue(path.getEdges().isEmpty());

        assertEquals(1, path.getVertices().size());
        assertEquals(v, path.getVertices().get(0));

        final Iterator i = path.iterator();
        assertEquals(v, i.next());
        assertFalse(i.hasNext());
    }

    public void testSimplePath() throws Exception {
        final Vertex v1 = new TestVertex();
        final Edge e = new TestEdge();
        final Vertex v2 = new TestVertex();
        final List pathList = new LinkedList();
        pathList.add(v1);
        pathList.add(e);
        pathList.add(v2);
        final Path path = new ListPath(pathList);

        assertEquals(v1, path.getStart());
        assertEquals(v2, path.getEnd());

        assertEquals(1, path.getEdges().size());
        assertEquals(e, path.getEdges().get(0));

        assertEquals(2, path.getVertices().size());
        assertEquals(v1, path.getVertices().get(0));
        assertEquals(v2, path.getVertices().get(1));

        final Iterator i = path.iterator();
        assertEquals(v1, i.next());
        assertEquals(e, i.next());
        assertEquals(v2, i.next());
        assertFalse(i.hasNext());
    }

}